/*
 * arch/arm/mach-ixp4xx/ng-setup.c
 *
 * Netgear FVX538/FVS338 board-setup 
 *
 * Copyright (c) 2006, TeamF1, Inc. 
 * Copyright (C) 2003-2004 MontaVista Software, Inc.
 *
 * Author: Deepak Saxena <dsaxena@plexity.net>
 */

#include <linux/init.h>
#include <linux/device.h>
#include <linux/serial.h>
#include <linux/tty.h>
#include <linux/serial_core.h>

#include <asm/types.h>
#include <asm/setup.h>
#include <asm/memory.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/flash.h>

#ifdef  __ARMEB__
#define REG_OFFSET 3
#else
#define REG_OFFSET 0
#endif

/*
 * Only one serial port is connected on the FVX538/FVS338.
 */
static struct uart_port ng_serial_port = {
    .membase    = (char*)(IXP4XX_UART2_BASE_VIRT + REG_OFFSET),
    .mapbase    = (IXP4XX_UART2_BASE_PHYS),
    .irq        = IRQ_IXP4XX_UART2,
    .flags      = UPF_SKIP_TEST,
    .iotype     = UPIO_MEM,
    .regshift   = 2,
    .uartclk    = IXP4XX_UART_XTAL,
    .line       = 0,
    .type       = PORT_XSCALE,
    .fifosize   = 32
};

void __init ng_map_io(void)
{
    early_serial_setup(&ng_serial_port);

    ixp4xx_map_io();
}

static struct flash_platform_data ng_flash_data = {
    .map_name   = "cfi_probe",
    .width      = 2,
};

static struct resource ng_flash_resource = {
    .start      = NG_FLASH_BASE,
#ifdef CONFIG_ARCH_NG_FVX538
    .end        = NG_FLASH_BASE + NG_FVX538_FLASH_SIZE,
#endif
#ifdef CONFIG_ARCH_NG_FVS338
    .end        = NG_FLASH_BASE + NG_FVS338_FLASH_SIZE,
#endif
    .flags      = IORESOURCE_MEM,
};

static struct platform_device ng_flash = {
    .name           = "IXP4XX-Flash",
    .id             = 0,
    .dev            = {
        .platform_data = &ng_flash_data,
    },
    .num_resources  = 1,
    .resource       = &ng_flash_resource,
};

static struct platform_device *ng_devices[] __initdata = {
    &ng_flash
};

static void __init ng_init(void)
{
    platform_add_devices(ng_devices, ARRAY_SIZE(ng_devices));
}

#ifdef CONFIG_ARCH_NG_FVX538
MACHINE_START(NG_FVX538, "Netgear FVX538")
        MAINTAINER("Netgear")
        BOOT_MEM(PHYS_OFFSET, IXP4XX_PERIPHERAL_BASE_PHYS,
                IXP4XX_PERIPHERAL_BASE_VIRT)
        MAPIO(ng_map_io)
        INITIRQ(ixp4xx_init_irq)
        .timer = &ixp4xx_timer,
        BOOT_PARAMS(0x0100)
        INIT_MACHINE(ng_init)
MACHINE_END
#endif

#ifdef CONFIG_ARCH_NG_FVS338
MACHINE_START(NG_FVS338, "Netgear FVS338")
        MAINTAINER("Netgear")
        BOOT_MEM(PHYS_OFFSET, IXP4XX_PERIPHERAL_BASE_PHYS,
                IXP4XX_PERIPHERAL_BASE_VIRT)
        MAPIO(ng_map_io)
        INITIRQ(ixp4xx_init_irq)
        .timer = &ixp4xx_timer,
        BOOT_PARAMS(0x0100)
        INIT_MACHINE(ng_init)
MACHINE_END
#endif
